
import math

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from error_adj_ipm import error_adjusted_ipm
from generate_lp import construct_random_LP

'''
This code tests the relationship between the number of variables in an LP (n) and
the number of iterations.  Run to reproduce Figure 1 or 3 depending on SKETCH_SIZE.
'''

FILENAME = 'num_it_vs_n.png'     # Output file of plot
NUM_CONSTRAINTS = 20             # Number of rows in the constraint matrix 'A'
EPSILON = 0.1                    # Error tolerance of the IPM
REPS = 60                        # Number of times to repeat the experiment at each 'n'
SKETCH_SIZE = 0                  # Size of sketch used in iterative solver.  Set to zero for SolveV-perturb
N_list = [x**2 for x in range(7,20)]  # List of values of 'n' to test.
delta_func = lambda n: 0.001     # Precision of linear solver as a function of 'n'


''' Test primal feasibility dependency on sqrt(n) '''

result_list = []

for N_dim in N_list:
    print("Current N:", N_dim)
    for rep_it in range(REPS):
        print("Rep:", rep_it + 1)
        
        A_mat, initial_point = construct_random_LP(NUM_CONSTRAINTS, N_dim)
        results = error_adjusted_ipm(A_mat, initial_point, EPSILON, 
                                     delta_func(N_dim), sketch_size = SKETCH_SIZE)
        
        result_list.append([N_dim,
                            results['num_iterations'],
                            results['primal_infeasibilty']])
        
result_df = pd.DataFrame(result_list)
result_df.columns = ['N_val', 'num_it', 'primal_infeas']
result_df = result_df.groupby('N_val').quantile([0.1, 0.5, 0.9]).unstack()['num_it'].reset_index()

trans_df = result_df.copy()

trans_df['N_val'] = trans_df['N_val'].apply(lambda x: x**(1/2))

plt.xlabel("sqrt(n)")
plt.ylabel("Number iterations")

plt.plot(trans_df['N_val'], trans_df[0.5], 'ro-', markersize = 5)
plt.fill_between(trans_df['N_val'], trans_df[0.1], trans_df[0.9], color='blue', alpha=0.5)

plt.savefig(FILENAME, dpi=300)

